/*
 * Created on 10.11.2004
 *
 * SVN header information:
 *  $Author: LBST-PF-3\orahn $
 *  $Rev: 2509 $
 *  $Date: 2006-10-06 12:01:50 +0200 (Fr, 06 Okt 2006) $
 *  $Id: PirolCSV_Parser.java 2509 2006-10-06 10:01:50Z LBST-PF-3\orahn $
 */
package de.fhOsnabrueck.jump.pirol.datasources.pirolCSV;

import java.io.BufferedReader;
import java.io.IOException;
import java.security.InvalidParameterException;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.TreeSet;

import com.vividsolutions.jump.feature.AttributeType;

import de.fhOsnabrueck.jump.pirol.utilities.AsciiFileParsing.CsvRawPointDataset;
import de.fhOsnabrueck.jump.pirol.utilities.AsciiFileParsing.DatasetPerRowCsvParser;
import de.fhOsnabrueck.jump.pirol.utilities.FeatureCollection.PirolFeatureCollection;
import de.fhOsnabrueck.jump.pirol.utilities.FeatureCollection.RolePointFeatures;
import de.fhOsnabrueck.jump.pirol.utilities.attributes.AttributeInfo;
import de.fhOsnabrueck.jump.pirol.utilities.debugOutput.DebugUserIds;
import de.fhOsnabrueck.jump.pirol.utilities.debugOutput.PersonalLogger;


/**
 * Class to parse an ASCII-File, containing one Dataset per row with the three PIROL-header-lines on top of the data.
 * Valid separators for a file to be parsed are listed in the field  <code>validSepatators</code>.
 * <br>
 * <b>Warning: For some reason a ClassDefNotFound-Exception is thrown, if this PlugIn is loaded from a Jar-file under Windows and a PersonalLogger is used in this class - I just have no idea, why!! Works great under Linux.</b>
 * <br>Note: Seems to be related to class loader problems. If the baseClasses are added to the classpath, the problem disappears...
 * 
 * @author Ole Rahn
 * <br>
 * <br>FH Osnabr&uuml;ck - University of Applied Sciences Osnabr&uuml;ck,
 * <br>Project: PIROL (2005),
 * <br>Subproject: Daten- und Wissensmanagement
 * 
 * @version $Rev: 2509 $
 * @see de.fhOsnabrueck.jump.pirol.utilities.AsciiFileParsing.DatasetPerRowCsvParser
 */
public class PirolCSV_Parser {
	
    public static final String DATATYPE_DOUBLE = "double";
    public static final String DATATYPE_INT = "int";
    public static final String DATATYPE_STRING = "string";
    /**
     * not used, yet
     */
    public static final String DATATYPE_DATE = "date";
	
	public static final String[] validSepatators = DatasetPerRowCsvParser.validSepatators;
	public static final String[] validSepatatorNames = DatasetPerRowCsvParser.validSepatatorNames;
	
	protected int xColumn, yColumn, zColumn;
	
	private BufferedReader buffReader = null;
	
	protected String separator = null;
	
	protected TreeSet tableColumns = null;
	
	private boolean headerRead = false, dataRead = false, parsed = false;
	
	// can not use logger here for some mysterious reason...
	protected PersonalLogger logger = null;
    
    protected DatasetPerRowCsvParser dataParser = null;
	
	public PirolCSV_Parser( BufferedReader isReader ) throws InvalidParameterException, NoSuchElementException, IOException{
		this.buffReader = isReader;
		this.buffReader.mark(2048);
		this.tableColumns = new TreeSet();
		
		xColumn = 1;
		yColumn = 0;
		zColumn = -1;
        
		this.logger = new PersonalLogger(DebugUserIds.OLE);
	}

	/**
	 * @return Returns if the file was parsed.
	 */
	public boolean isParsed() {
		return parsed;
	}
	public String getSeparator() {
		return separator;
	}
 
	public void setSeparator(String separator) {
		if ( separator != null)
			this.separator = separator;
	}

	public void setXColumn(int column) {
		this.xColumn = column;
	}

	public void setYColumn(int column) {
		this.yColumn = column;
	}
	
	public AttributeInfo[] getColumns() throws InvalidParameterException, NoSuchElementException, IOException{
		if ( !this.headerRead ) this.parse();
		
		Object[] objs = this.tableColumns.toArray();
        AttributeInfo[] cols = new AttributeInfo[objs.length];
		
		for ( int i=0; i<objs.length; i++){
			cols[i] = (AttributeInfo)objs[i];
		}
		
		return cols;
	}
	
	
	public AttributeInfo getColumn( int nr ) throws InvalidParameterException, NoSuchElementException, IOException{
        AttributeInfo[] cols = this.getColumns();
		if ( cols.length > nr )
			return cols[nr];
		
		return null;
	}
	
	public CsvRawPointDataset[] getFeaturesArray() throws InvalidParameterException, NoSuchElementException, IOException{
        if (!this.parsed) this.parse();
		return this.dataParser.getFeaturesArray();
	}
	
	public List getFeaturesList() throws InvalidParameterException, NoSuchElementException, IOException{
        if (!this.parsed) this.parse();
		return this.dataParser.getFeaturesList();
	}
	
	public boolean isEmpty() throws InvalidParameterException, NoSuchElementException, IOException{
        if (!this.parsed) this.parse();
		return this.dataParser.isEmpty();
	}
	
	public CsvRawPointDataset getFeature( int nr ) throws InvalidParameterException, NoSuchElementException, IOException{
        if (!this.parsed) this.parse();
		return this.dataParser.getFeature(nr);
	}
	
	public int getNumOfDatasets() throws InvalidParameterException, NoSuchElementException, IOException{
        if (!this.parsed) this.parse();
		return this.dataParser.getNumOfDatasets();
	}
	
	public void parse() throws InvalidParameterException, NoSuchElementException, IOException {
		if ( !this.headerRead )
			this.headerRead = this.readHeader();
		
        if (this.dataParser == null){
            this.dataParser = new DatasetPerRowCsvParser(this.getColumns(), this.buffReader);
            this.dataParser.setXColumn(this.xColumn);
            this.dataParser.setYColumn(this.yColumn);
            this.dataParser.setSeparator(this.getSeparator());
        }
        
		if ( this.headerRead && !this.dataRead ){
			this.dataParser.parse();
            dataRead = this.dataParser.isDataRead();
		}
		
		this.parsed = true;
	}
	
	public void close() throws IOException{
		if ( this.buffReader != null )
			this.buffReader.close();
	}
	
    /**
     * reads the file header lines and puts the information in String arrays.
     *@return array of 3 String arrays containing the attribute names, the entities, the datatypes of the attributes.
     *@throws IOException
     *@throws NoSuchElementException if one of the lines doesn't seem to be a valid header line (does not beginn with <code>$</code>)
     */
	protected String[] readHeaderLines() throws IOException{
		String line;
		String[] lines = new String[3];
		for ( int i = 0; i<3 && (line=this.buffReader.readLine())!=null; i++ ){
			if ( line.charAt(0)!='$' ){
				throw new NoSuchElementException("line "+Integer.toString(i+1)+" is not a Pirol-CSV header line.");
			}
			line = line.replaceFirst("^[$]?","");

			lines[i] = line;
		}
		return lines;
	}
	
	public String[] getHeaderLinesForDialog() throws IOException{
		String[] result = this.readHeaderLines();
		this.buffReader.reset();
		return result;
	}
	
	protected boolean readHeader() throws IOException, NoSuchElementException, InvalidParameterException {
		
		String[] lines = this.readHeaderLines();
		
		if (this.separator==null)
			this.separator = DatasetPerRowCsvParser.guessSepatator(lines[2]);
		
		AttributeInfo[] cols = this.extractColumns(lines[0].split(this.separator),lines[1].split(this.separator),lines[2].split(this.separator));
		
		if ( cols != null ){
			for ( int i = 0; i<cols.length; i++ ){
				this.tableColumns.add(cols[i]);
			}
			return true;
		}
		return false;
	}
	
    /**
     * extracts information about the columns and puts them info AttributeInfo objects.
     *@param names
     *@param entities
     *@param types
     *@return array of AttributeInfos
     *@throws InvalidParameterException if even the header lines have different numbers of fields
     */
	protected AttributeInfo[] extractColumns( String[] names, String[] entities, String[] types ) throws InvalidParameterException {
		if ( names.length != entities.length || names.length != types.length ){
			throw new InvalidParameterException("Pirol CSV header lines have differnt numbers of fields.");
		}
		
        AttributeInfo[] cols = new AttributeInfo[names.length];
        AttributeType at;
		
		for ( int i=0; i<names.length; i++){
            if ( types[i].trim().equals( PirolCSV_Parser.DATATYPE_DOUBLE ) )
                at = AttributeType.DOUBLE;
            else if ( types[i].trim().equals( PirolCSV_Parser.DATATYPE_INT ) )
                at = AttributeType.INTEGER;
            else
                at = AttributeType.STRING;
			cols[i] = new AttributeInfo(at, names[i].trim());
            cols[i].setUnitIdentifier( entities[i].trim() );
            cols[i].setIndex(i);
		}
		
		return cols;
	}

    public PirolFeatureCollection createFeatureCollection() throws Exception {
        if (!this.parsed) this.parse();
        return dataParser.createFeatureCollection(new RolePointFeatures());
    }
    
    
}
