/*
 * Created on 10.04.2006 for PIROL
 *
 * SVN header information:
 *  $Author: LBST-PF-3\orahn $
 *  $Rev: 2588 $
 *  $Date: 2006-10-24 10:36:51 +0200 (Di, 24 Okt 2006) $
 *  $Id: ReprojectionDescription.java 2588 2006-10-24 08:36:51Z LBST-PF-3\orahn $
 */
package de.fhOsnabrueck.jump.pirol.utilities.projection;

import de.fhOsnabrueck.jump.pirol.utilities.debugOutput.DebugUserIds;
import de.fhOsnabrueck.jump.pirol.utilities.debugOutput.PersonalLogger;
import de.fhOsnabrueck.jump.pirol.utilities.projection.Projections.AbstractProjection;
import de.fhOsnabrueck.jump.pirol.utilities.projection.Projections.GaussKruegerProjection;
import de.fhOsnabrueck.jump.pirol.utilities.projection.Projections.Wgs84Projection;
import de.fhOsnabrueck.jump.pirol.utilities.projection.ellipsoid.BesselEllipsoid;

/**
 * Class describing a cs and/or projection change
 *
 * @author Ole Rahn
 * <br>
 * <br>FH Osnabr&uuml;ck - University of Applied Sciences Osnabr&uuml;ck,
 * <br>Project: PIROL (2006),
 * <br>Subproject: Daten- und Wissensmanagement
 * 
 * @version $Rev: 2588 $
 * 
 */
public class ReprojectionDescription {

    protected AbstractProjection startProjection = null, targetProjection = null;
    
    protected boolean wgsToGK = false, gkToWgs = false, gkMeridianBandChange = false;
    
    protected static PersonalLogger logger = new PersonalLogger(DebugUserIds.OLE);

    public ReprojectionDescription(AbstractProjection startProjection, AbstractProjection targetProjection) {
        super();
        
        if (startProjection==null || targetProjection==null){
            throw new IllegalArgumentException("source or target projection is null: " + "(" + startProjection + ", " + targetProjection + ")");
        }
        
        this.startProjection = startProjection;
        this.targetProjection = targetProjection;
        
        this.wgsToGK = Wgs84Projection.class.isInstance(startProjection) && GaussKruegerProjection.class.isInstance(targetProjection);
        this.gkToWgs = GaussKruegerProjection.class.isInstance(startProjection) && Wgs84Projection.class.isInstance(targetProjection);
        this.gkMeridianBandChange = !startProjection.equals(targetProjection) && GaussKruegerProjection.class.isInstance(startProjection) && GaussKruegerProjection.class.isInstance(targetProjection);
    }

    public AbstractProjection getStartProjection() {
        return startProjection;
    }

    public AbstractProjection getTargetProjection() {
        return targetProjection;
    }
    
    public boolean needHelmertTransformation(){
        return ( (BesselEllipsoid.class.isInstance(this.startProjection.getEllipsoid()) || BesselEllipsoid.class.isInstance(this.targetProjection.getEllipsoid())) && 
                (!BesselEllipsoid.class.isInstance(this.startProjection.getEllipsoid()) || !BesselEllipsoid.class.isInstance(this.targetProjection.getEllipsoid())) );
    }

    public boolean isGaussKruegerToWgs84() {
        return gkToWgs;
    }

    public boolean isWgs84ToGaussKrueger() {
        return wgsToGK;
    }
    
    public boolean isGkMeridianBandChange() {
        return gkMeridianBandChange;
    }

    public boolean startProjectionEqualsTargetProjection(){
        return this.startProjection.equals(this.targetProjection);
    }
    
    /**
     * I assume -like in our database - that WGS84 is the basic coordinate configuration.
     * All other projections and coordinate systems that need a different ellipsoid have to
     * deliver Helmert parameters for the transformation. In our case this is only the
     * Gauss-Krger projection which in on a Bessel ellisoid instead of a GRS80.<br><br>
     * This method gives the helmert parameter dependend projection or null if there is no such projection.  
     */
    public GaussKruegerProjection getProjectionWithHelmertParameter(){
        if (GaussKruegerProjection.class.isInstance(this.getStartProjection()))
            return (GaussKruegerProjection)this.getStartProjection();
        else if (GaussKruegerProjection.class.isInstance(this.getTargetProjection()))
            return (GaussKruegerProjection)this.getTargetProjection();
        
        return null;
    }
    
    public static ReprojectionDescription createInstance(int sridStart, int sridTarget){
    	logger.printDebug("creating reprojection for: " + sridStart + ", " + sridTarget);
        return new ReprojectionDescription(AbstractProjection.createInstance(sridStart), AbstractProjection.createInstance(sridTarget));
    }

    public String toString() {
        return "Reprojection: " + this.startProjection.toString() + " -> " + this.targetProjection.toString();
    }
    
    public static ReprojectionDescription getReverseProjection(ReprojectionDescription reprojectionDescription){
    	return ReprojectionDescription.createInstance(reprojectionDescription.getTargetProjection().getSRID(), reprojectionDescription.getStartProjection().getSRID());
    }
    
    /**
     * 
     * @return true if source and target projection are equal, false if not
     */
    public boolean givenProjectionsAreEqual(){
    	return this.getStartProjection().getSRID() == this.getTargetProjection().getSRID();
    }

}
